/*******************************************************************************
 * Copyright Moxa Inc.
 *
 * TAG Service for SNMP Trap Service Application
 *
 * Date          Author       Comment
 * 01-26-2016    TJ Tai       Created.
 ******************************************************************************/

/*!
    \file ioPAC8600_snmp_trap_service.c
    \copyright Moxa Inc.
    \brief <b>\a ioPAC8600 \a SNMP \a Trap \a Application \a TAG \a Sample</b>
    \date 01-26-2016
    \author TJ Tai
    \version V1.0
    \image html 8600_snmp_trap.jpg "SNMP Trap Service Sample"
    \details
    <pre>
    <b>Introduction:</b>
    Check if DI0 status is enable, then send snmp trap to the specific IP
    Please use mib browser trap receiver to receive snmp trap from device
    <b>Example:</b>
    ./ioPAC8600_snmp_trap_service 
    <b>Help:</b>
    </pre>
    \verbatim
    root@Moxa:/tmp#./ioPAC8600_snmp_trap_service -h
    SNMP trap service program.

    Usage: ./ioPAC8600_snmp_trap_service [OPTIONS]

    Options:
        -i        Slot of DI module [1-9]. Using with -i. Default value: 1
                  Ex. ./ioPAC8600_snmp_trap_service -i 1
    \endverbatim
    \b Library: \n
    \link TagService
    TAG APIs
    \endlink
    \n\b RTUxpress \b Project \b file: \n
    <a href="./rtu/ioPAC8600_snmp_trap_service.rtu" target="_blank"><b>ioPAC8600_snmp_trap_service.rtu</b></a>
    \n(Please right click on the link and ‘Save Target As…’ to save RTUxpress project file and open it with RTUxpress utility)
*/

#include "libmoxa_rtu.h"

/*******************************************************************************
 *
 * Sample code for TAG SNMP Trap Service
 *
 *******************************************************************************/
void usage(void)
{
	printf("%s.\n\n", "SNMP trap service Program");
	printf("Usage: ./ioPAC8600_snmp_trap_service [OPTIONS]\n\n");
	printf("Options:\n");
	printf("\t%-8s Slot of DI module [1-9]. Using with -i. Default value: 1 \n", "-i");
	printf("\t         Ex. ./ioPAC8600_snmp_trap_service -i 1\n");
	printf("\n");
}

//! \include ioPAC8600_snmp_trap_service.c
int main(int argc, char * argv[])
{
	int rc;
	TAG_ERR_CODE retval = 0;
	TAG_INFO tagInfo;
	UINT32 diSlot = 1, slotMin = 0, slotMax = 9;
	int diChannelAmount = 16;
	char di_tagName[diChannelAmount][TAG_MAX_NAME_SIZE];
	int i, j;
	UINT32 u32Val = 0;
	INT32 bitVal = 0;
	INT32 ch00Val = 0;
    int chStatus[1];
	char trapServiceStatus[1][TAG_MAX_NAME_SIZE];
	char trapTrigger[1][TAG_MAX_NAME_SIZE];

	while(-1 != (rc = getopt(argc, argv, "i:h")))
	{
		switch(rc)
		{
			case 'i':
				diSlot = atoi(optarg);
				if(diSlot < slotMin || diSlot > slotMax)
				{
					printf("Error parameter: slot: %d\n", diSlot);
					return -1;
				}
				break;
			case 'h':
			case '?':
			default:
				usage();
				return 0;
		}
	}

    sprintf(trapServiceStatus[0], "Service_SNMPAlarm_Status");
    sprintf(trapTrigger[0], "Service_SNMPAlarm_Trigger_0");
	printf("%-10s: %d\n", "DI slot", diSlot);

	retval = MX_RTU_Tag_Init();
	if(retval != TAG_ERR_OK)
	{
		printf("MX_RTU_Tag_Init(), return code = %d.\n", retval);
		return 0;
	}

	while(1)
	{
		// Check Service Init
		retval = MX_RTU_Tag_Read(trapServiceStatus[0], &u32Val, sizeof(u32Val), NULL, NULL);
		if(retval != TAG_ERR_OK)
		{
			printf("MX_RTU_Tag_Read(%s) = %d\n", trapServiceStatus[0], retval);
		}
		if(u32Val == STATUS_RUNNING)
		{
			printf("SNMP Trap Service Init Success.\n");
			break;
		}
		else if(u32Val == STATUS_FAIL)
		{
			printf("SNMP Trap Service Init fail.\n");
			return 0;
		}
		else if(u32Val == STATUS_INIT)
		{
			printf("SNMP Trap Service Initing, please wait ...\r\n");
			sleep(5);
		}
	}

	// Config DI TAG
	for(i = 0; i < diChannelAmount; i++)
	{
		sprintf(di_tagName[i], "S%d_DI%d_DIValue", diSlot, i);
	}

	while(1)
	{

		retval = MX_RTU_Tag_Get_Info(di_tagName[0], &tagInfo);
		if(retval != TAG_ERR_OK)
		{
			printf("MX_RTU_Tag_Get_Info(%s), return code = %d\n", di_tagName[0], retval);
			break;
		}

        retval = MX_RTU_Tag_Read(di_tagName[0], &ch00Val, sizeof(ch00Val), NULL, NULL);

        if (ch00Val == 1) {
            // send trap by trigger snmp trap trigger tag
            printf("send trap to specific IP\n");
            retval = MX_RTU_Tag_Get_Info(trapTrigger[0], &tagInfo);
            if(retval != TAG_ERR_OK)
            {
                printf("MX_RTU_Tag_Get_Info(%s), return code = %d.\n", trapTrigger[0], retval);
                return 0;
            }
            retval = MX_RTU_Tag_Write(tagInfo.tagName, &ch00Val, tagInfo.tagSize);
            if(retval != TAG_ERR_OK)
            {
                printf("MX_RTU_Tag_Write(%s), return code = %d\r\n", tagInfo.tagName, retval);
                return 0;
            }
            
            
        }

		sleep(1);
	}

	retval = MX_RTU_Tag_Uninit();
	if(retval != TAG_ERR_OK)
	{
		printf("MX_RTU_Tag_Uninit(), return code = %d\n", retval);
	}

	return 0;
}
